@@ -17,7 +17,7 @@
 %                                March 2008                                   %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2009 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999-2010 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
@@ -116,7 +116,7 @@
 */
 typedef struct _DDSPixelFormat
 {
-  unsigned long
+  size_t
     flags,
     fourcc,
     rgb_bitcount,
@@ -128,13 +128,13 @@ typedef struct _DDSPixelFormat
 
 typedef struct _DDSInfo
 {
-	unsigned long
-		flags,
-		height,
-		width,
-		pitchOrLinearSize,
-		depth,
-		mipmapcount,
+  size_t
+    flags,
+    height,
+    width,
+    pitchOrLinearSize,
+    depth,
+    mipmapcount,
     ddscaps1,
     ddscaps2;
   
@@ -225,7 +225,7 @@ static void
 %
 */
 
-static inline unsigned long Min(unsigned long one, unsigned long two)
+static inline size_t Min(size_t one, size_t two)
 {
   if (one < two)
     return one;
@@ -247,12 +247,12 @@ static Image *ReadDDSImage(const ImageInfo *image_info,ExceptionInfo *exception)
     compression;
 
   DDSInfo
-  	dds_info;
+    dds_info;
   
   DDSDecoder
     *decoder;
   
-  unsigned long
+  size_t
     n, num_images;
   
   /*
@@ -277,7 +277,7 @@ static Image *ReadDDSImage(const ImageInfo *image_info,ExceptionInfo *exception)
     Initialize image structure.
   */
   if (ReadDDSInfo(image, &dds_info) != MagickTrue) {
-	  ThrowReaderException(CorruptImageError,"ImproperImageHeader");
+    ThrowReaderException(CorruptImageError,"ImproperImageHeader");
   }
   
   if (dds_info.ddscaps2 & DDSCAPS2_CUBEMAP)
@@ -408,33 +408,33 @@ static Image *ReadDDSImage(const ImageInfo *image_info,ExceptionInfo *exception)
 
 static MagickBooleanType ReadDDSInfo(Image *image, DDSInfo *dds_info)
 {
-	unsigned long
+  size_t
     hdr_size,
     required;
-	
-	/* Seek to start of header */
-	(void) SeekBlob(image, 4, SEEK_SET);
-	
-	/* Check header field */
-	hdr_size = ReadBlobLSBLong(image);
-	if (hdr_size != 124)
-		return MagickFalse;
-	
-	/* Fill in DDS info struct */
-	dds_info->flags = ReadBlobLSBLong(image);
   
-	/* Check required flags */
-	required=(unsigned long) (DDSD_WIDTH | DDSD_HEIGHT | DDSD_PIXELFORMAT);
+  /* Seek to start of header */
+  (void) SeekBlob(image, 4, SEEK_SET);
+  
+  /* Check header field */
+  hdr_size = ReadBlobLSBLong(image);
+  if (hdr_size != 124)
+    return MagickFalse;
+  
+  /* Fill in DDS info struct */
+  dds_info->flags = ReadBlobLSBLong(image);
+  
+  /* Check required flags */
+  required=(size_t) (DDSD_WIDTH | DDSD_HEIGHT | DDSD_PIXELFORMAT);
   if ((dds_info->flags & required) != required)
-		return MagickFalse;
+    return MagickFalse;
   
-	dds_info->height = ReadBlobLSBLong(image);
-	dds_info->width = ReadBlobLSBLong(image);
-	dds_info->pitchOrLinearSize = ReadBlobLSBLong(image);
-	dds_info->depth = ReadBlobLSBLong(image);
-	dds_info->mipmapcount = ReadBlobLSBLong(image);
+  dds_info->height = ReadBlobLSBLong(image);
+  dds_info->width = ReadBlobLSBLong(image);
+  dds_info->pitchOrLinearSize = ReadBlobLSBLong(image);
+  dds_info->depth = ReadBlobLSBLong(image);
+  dds_info->mipmapcount = ReadBlobLSBLong(image);
   
-	(void) SeekBlob(image, 44, SEEK_CUR); 	/* reserved region of 11 DWORDs */
+  (void) SeekBlob(image, 44, SEEK_CUR);   /* reserved region of 11 DWORDs */
   
   /* Read pixel format structure */
   hdr_size = ReadBlobLSBLong(image);
@@ -449,11 +449,11 @@ static MagickBooleanType ReadDDSInfo(Image *image, DDSInfo *dds_info)
   dds_info->pixelformat.b_bitmask = ReadBlobLSBLong(image);
   dds_info->pixelformat.alpha_bitmask = ReadBlobLSBLong(image);
   
-	dds_info->ddscaps1 = ReadBlobLSBLong(image);
-	dds_info->ddscaps2 = ReadBlobLSBLong(image);
-	(void) SeekBlob(image, 12, SEEK_CUR); /* 3 reserved DWORDs */
+  dds_info->ddscaps1 = ReadBlobLSBLong(image);
+  dds_info->ddscaps2 = ReadBlobLSBLong(image);
+  (void) SeekBlob(image, 12, SEEK_CUR); /* 3 reserved DWORDs */
   
-	return MagickTrue;
+  return MagickTrue;
 }
 
 static void CalculateColors(unsigned short c0, unsigned short c1,
@@ -498,35 +498,35 @@ static MagickBooleanType ReadDXT1(Image *image, DDSInfo *dds_info)
   ExceptionInfo
     *exception;
 
-  long
+  ssize_t
     j,
     y;
-	
+  
   PixelPacket
     *q;
   
-  register long
+  register ssize_t
     i,
     x;
   
   unsigned char
     code;
   
-	unsigned short
+  unsigned short
     c0,
     c1;
-	
-  unsigned long
+  
+  size_t
     bits;
 
   exception=(&image->exception);
-  for (y = 0; y < (long) dds_info->height; y += 4)
+  for (y = 0; y < (ssize_t) dds_info->height; y += 4)
   {
-    for (x = 0; x < (long) dds_info->width; x += 4)
+    for (x = 0; x < (ssize_t) dds_info->width; x += 4)
     {
       /* Get 4x4 patch of pixels to write on */
       q = QueueAuthenticPixels(image, x, y, Min(4, dds_info->width - x),
-	                     Min(4, dds_info->height - y),exception);
+                       Min(4, dds_info->height - y),exception);
       
       if (q == (PixelPacket *) NULL)
         return MagickFalse;
@@ -543,7 +543,7 @@ static MagickBooleanType ReadDXT1(Image *image, DDSInfo *dds_info)
       {
         for (i = 0; i < 4; i++)
         {
-          if ((x + i) < (long) dds_info->width && (y + j) < (long) dds_info->height)
+          if ((x + i) < (ssize_t) dds_info->width && (y + j) < (ssize_t) dds_info->height)
             {
               code = (unsigned char) ((bits >> ((j*4+i)*2)) & 0x3);
               q->red     = ScaleCharToQuantum( colors.r[code] );
@@ -576,21 +576,21 @@ static MagickBooleanType ReadDXT3(Image *image, DDSInfo *dds_info)
   ExceptionInfo
     *exception;
 
-  long
+  ssize_t
     j,
     y;
 
   PixelPacket
     *q;
   
-  register long
+  register ssize_t
     i,
     x;
   
   unsigned char
     alpha;
   
-  unsigned long
+  size_t
     a0,
     a1,
     bits,
@@ -601,9 +601,9 @@ static MagickBooleanType ReadDXT3(Image *image, DDSInfo *dds_info)
     c1;
   
   exception=(&image->exception);
-  for (y = 0; y < (long) dds_info->height; y += 4)
+  for (y = 0; y < (ssize_t) dds_info->height; y += 4)
   {
-    for (x = 0; x < (long) dds_info->width; x += 4)
+    for (x = 0; x < (ssize_t) dds_info->width; x += 4)
     {
       /* Get 4x4 patch of pixels to write on */
       q = QueueAuthenticPixels(image, x, y, Min(4, dds_info->width - x),
@@ -628,7 +628,7 @@ static MagickBooleanType ReadDXT3(Image *image, DDSInfo *dds_info)
       {
         for (i = 0; i < 4; i++)
         {
-          if ((x + i) < (long) dds_info->width && (y + j) < (long) dds_info->height)
+          if ((x + i) < (ssize_t) dds_info->width && (y + j) < (ssize_t) dds_info->height)
             {
               code = (bits >> ((4*j+i)*2)) & 0x3;
               q->red   = ScaleCharToQuantum( colors.r[code] );
@@ -668,7 +668,7 @@ static MagickBooleanType ReadDXT5(Image *image, DDSInfo *dds_info)
   ExceptionInfo
     *exception;
 
-  long
+  ssize_t
     j,
     y;
 
@@ -678,7 +678,7 @@ static MagickBooleanType ReadDXT5(Image *image, DDSInfo *dds_info)
   PixelPacket
     *q;
   
-  register long
+  register ssize_t
     i,
     x;
 
@@ -686,7 +686,7 @@ static MagickBooleanType ReadDXT5(Image *image, DDSInfo *dds_info)
     a0,
     a1;
   
-  unsigned long
+  size_t
     alpha,
     bits,
     code,
@@ -697,9 +697,9 @@ static MagickBooleanType ReadDXT5(Image *image, DDSInfo *dds_info)
     c1;
   
   exception=(&image->exception);
-  for (y = 0; y < (long) dds_info->height; y += 4)
+  for (y = 0; y < (ssize_t) dds_info->height; y += 4)
   {
-    for (x = 0; x < (long) dds_info->width; x += 4)
+    for (x = 0; x < (ssize_t) dds_info->width; x += 4)
     {
       /* Get 4x4 patch of pixels to write on */
       q = QueueAuthenticPixels(image, x, y, Min(4, dds_info->width - x),
@@ -727,7 +727,7 @@ static MagickBooleanType ReadDXT5(Image *image, DDSInfo *dds_info)
       {
         for (i = 0; i < 4; i++)
         {
-          if ((x + i) < (long) dds_info->width && (y + j) < (long) dds_info->height)
+          if ((x + i) < (ssize_t) dds_info->width && (y + j) < (ssize_t) dds_info->height)
             {
               code = (bits >> ((4*j+i)*2)) & 0x3;
               q->red   = ScaleCharToQuantum( colors.r[code] );
@@ -735,7 +735,7 @@ static MagickBooleanType ReadDXT5(Image *image, DDSInfo *dds_info)
               q->blue  = ScaleCharToQuantum( colors.b[code] );
               
               /* Extract alpha value */
-              alpha_code = (unsigned long) (alpha_bits >> (3*(4*j+i))) & 0x7;
+              alpha_code = (size_t) (alpha_bits >> (3*(4*j+i))) & 0x7;
               if (alpha_code == 0)
                 alpha = a0;
               else if (alpha_code == 1)
@@ -771,25 +771,27 @@ static MagickBooleanType ReadUncompressedRGB(Image *image, DDSInfo *dds_info)
   ExceptionInfo
     *exception;
 
-  long
+  ssize_t
     x, y;
   
   PixelPacket
     *q;
   
   exception=(&image->exception);
-  for (y = 0; y < (long) dds_info->height; y++)
+  for (y = 0; y < (ssize_t) dds_info->height; y++)
   {
     q = QueueAuthenticPixels(image, 0, y, dds_info->width, 1,exception);
     
     if (q == (PixelPacket *) NULL)
       return MagickFalse;
     
-    for (x = 0; x < (long) dds_info->width; x++)
+    for (x = 0; x < (ssize_t) dds_info->width; x++)
     {
       q->blue  = ScaleCharToQuantum( (unsigned char) ReadBlobByte(image) );
       q->green = ScaleCharToQuantum( (unsigned char) ReadBlobByte(image) );
       q->red   = ScaleCharToQuantum( (unsigned char) ReadBlobByte(image) );
+      if (dds_info->pixelformat.rgb_bitcount == 32)
+        (void) ReadBlobByte(image);
       q++;
     }
     
@@ -807,21 +809,21 @@ static MagickBooleanType ReadUncompressedRGBA(Image *image, DDSInfo *dds_info)
   ExceptionInfo
     *exception;
 
-  long
+  ssize_t
     x, y;
   
   PixelPacket
     *q;
   
   exception=(&image->exception);
-  for (y = 0; y < (long) dds_info->height; y++)
+  for (y = 0; y < (ssize_t) dds_info->height; y++)
   {
     q = QueueAuthenticPixels(image, 0, y, dds_info->width, 1,exception);
     
     if (q == (PixelPacket *) NULL)
       return MagickFalse;
     
-    for (x = 0; x < (long) dds_info->width; x++)
+    for (x = 0; x < (ssize_t) dds_info->width; x++)
     {
       q->blue    = ScaleCharToQuantum( (unsigned char) ReadBlobByte(image) );
       q->green   = ScaleCharToQuantum( (unsigned char) ReadBlobByte(image) );
@@ -844,13 +846,13 @@ static MagickBooleanType ReadUncompressedRGBA(Image *image, DDSInfo *dds_info)
 */
 static void SkipDXTMipmaps(Image *image, DDSInfo *dds_info, int texel_size)
 {
-  register long
+  register ssize_t
     i;
 
   MagickOffsetType
     offset;
 
-  unsigned long
+  size_t
     h,
     w;
   
@@ -867,7 +869,7 @@ static void SkipDXTMipmaps(Image *image, DDSInfo *dds_info, int texel_size)
       /*
         Mipmapcount includes the main image, so start from one
       */
-      for (i = 1; (i < (long) dds_info->mipmapcount) && w && h; i++)
+      for (i = 1; (i < (ssize_t) dds_info->mipmapcount) && w && h; i++)
       {
         offset = (MagickOffsetType) ((w + 3) / 4) * ((h + 3) / 4) * texel_size;
         (void) SeekBlob(image, offset, SEEK_CUR);
@@ -886,10 +888,10 @@ static void SkipRGBMipmaps(Image *image, DDSInfo *dds_info, int pixel_size)
   MagickOffsetType
     offset;
   
-  register long
+  register ssize_t
     i;
 
-  unsigned long
+  size_t
     h,
     w;
 
@@ -906,7 +908,7 @@ static void SkipRGBMipmaps(Image *image, DDSInfo *dds_info, int pixel_size)
       /*
         Mipmapcount includes the main image, so start from one
       */
-      for (i=1; (i < (long) dds_info->mipmapcount) && w && h; i++)
+      for (i=1; (i < (ssize_t) dds_info->mipmapcount) && w && h; i++)
       {
         offset = (MagickOffsetType) w * h * pixel_size;
         (void) SeekBlob(image, offset, SEEK_CUR);
@@ -974,7 +976,7 @@ static MagickBooleanType IsDDS(const unsigned char *magick,const size_t length)
 %      RegisterDDSImage(void)
 %
 */
-ModuleExport unsigned long RegisterDDSImage(void)
+ModuleExport size_t RegisterDDSImage(void)
 {
   MagickInfo
     *entry;
