@@ -102,7 +102,7 @@ ModuleExport size_t RegisterHISTOGRAMImage(void)
   entry=SetMagickInfo("HISTOGRAM");
   entry->encoder=(EncodeImageHandler *) WriteHISTOGRAMImage;
   entry->adjoin=MagickFalse;
-  entry->format_type=ExplicitFormatType;
+  entry->format_type=ImplicitFormatType;
   entry->description=ConstantString("Histogram of the image");
   entry->module=ConstantString("HISTOGRAM");
   (void) RegisterMagickInfo(entry);
@@ -177,7 +177,7 @@ static inline size_t MagickMax(const size_t x,const size_t y)
 static MagickBooleanType WriteHISTOGRAMImage(const ImageInfo *image_info,
   Image *image)
 {
-#define HistogramDensity  "256x200"
+#define HistogramDensity  "256x100"
 
   ChannelType
     channel;
