@@ -123,7 +123,7 @@ MagickExport RectangleInfo GetImageBoundingBox(const Image *image,
   CacheView
     *image_view;
 
-  long
+  ssize_t
     y;
 
   MagickBooleanType
@@ -145,8 +145,8 @@ MagickExport RectangleInfo GetImageBoundingBox(const Image *image,
     (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",image->filename);
   bounds.width=0;
   bounds.height=0;
-  bounds.x=(long) image->columns;
-  bounds.y=(long) image->rows;
+  bounds.x=(ssize_t) image->columns;
+  bounds.y=(ssize_t) image->rows;
   GetMagickPixelPacket(image,&target[0]);
   image_view=AcquireCacheView(image);
   p=GetCacheViewVirtualPixels(image_view,0,0,1,1,exception);
@@ -158,12 +158,13 @@ MagickExport RectangleInfo GetImageBoundingBox(const Image *image,
   SetMagickPixelPacket(image,p,GetCacheViewAuthenticIndexQueue(image_view),
     &target[0]);
   GetMagickPixelPacket(image,&target[1]);
-  p=GetCacheViewVirtualPixels(image_view,(long) image->columns-1,0,1,1,
+  p=GetCacheViewVirtualPixels(image_view,(ssize_t) image->columns-1,0,1,1,
     exception);
   SetMagickPixelPacket(image,p,GetCacheViewAuthenticIndexQueue(image_view),
     &target[1]);
   GetMagickPixelPacket(image,&target[2]);
-  p=GetCacheViewVirtualPixels(image_view,0,(long) image->rows-1,1,1,exception);
+  p=GetCacheViewVirtualPixels(image_view,0,(ssize_t) image->rows-1,1,1,
+    exception);
   SetMagickPixelPacket(image,p,GetCacheViewAuthenticIndexQueue(image_view),
     &target[2]);
   status=MagickTrue;
@@ -171,7 +172,7 @@ MagickExport RectangleInfo GetImageBoundingBox(const Image *image,
 #if defined(MAGICKCORE_OPENMP_SUPPORT)
   #pragma omp parallel for schedule(dynamic,4) shared(status)
 #endif
-  for (y=0; y < (long) image->rows; y++)
+  for (y=0; y < (ssize_t) image->rows; y++)
   {
     MagickPixelPacket
       pixel;
@@ -185,7 +186,7 @@ MagickExport RectangleInfo GetImageBoundingBox(const Image *image,
     register const PixelPacket
       *restrict p;
 
-    register long
+    register ssize_t
       x;
 
     if (status == MagickFalse)
@@ -202,21 +203,21 @@ MagickExport RectangleInfo GetImageBoundingBox(const Image *image,
       }
     indexes=GetCacheViewVirtualIndexQueue(image_view);
     pixel=zero;
-    for (x=0; x < (long) image->columns; x++)
+    for (x=0; x < (ssize_t) image->columns; x++)
     {
       SetMagickPixelPacket(image,p,indexes+x,&pixel);
       if ((x < bounding_box.x) &&
           (IsMagickColorSimilar(&pixel,&target[0]) == MagickFalse))
         bounding_box.x=x;
-      if ((x > (long) bounding_box.width) &&
+      if ((x > (ssize_t) bounding_box.width) &&
           (IsMagickColorSimilar(&pixel,&target[1]) == MagickFalse))
-        bounding_box.width=(unsigned long) x;
+        bounding_box.width=(size_t) x;
       if ((y < bounding_box.y) &&
           (IsMagickColorSimilar(&pixel,&target[0]) == MagickFalse))
         bounding_box.y=y;
-      if ((y > (long) bounding_box.height) &&
+      if ((y > (ssize_t) bounding_box.height) &&
           (IsMagickColorSimilar(&pixel,&target[2]) == MagickFalse))
-        bounding_box.height=(unsigned long) y;
+        bounding_box.height=(size_t) y;
       p++;
     }
 #if defined(MAGICKCORE_OPENMP_SUPPORT)
@@ -260,8 +261,8 @@ MagickExport RectangleInfo GetImageBoundingBox(const Image *image,
 %
 %  The format of the GetImageChannelDepth method is:
 %
-%      unsigned long GetImageDepth(const Image *image,ExceptionInfo *exception)
-%      unsigned long GetImageChannelDepth(const Image *image,
+%      size_t GetImageDepth(const Image *image,ExceptionInfo *exception)
+%      size_t GetImageChannelDepth(const Image *image,
 %        const ChannelType channel,ExceptionInfo *exception)
 %
 %  A description of each parameter follows:
@@ -273,29 +274,27 @@ MagickExport RectangleInfo GetImageBoundingBox(const Image *image,
 %    o exception: return any errors or warnings in this structure.
 %
 */
-
-MagickExport unsigned long GetImageDepth(const Image *image,
-  ExceptionInfo *exception)
+MagickExport size_t GetImageDepth(const Image *image,ExceptionInfo *exception)
 {
   return(GetImageChannelDepth(image,AllChannels,exception));
 }
 
-MagickExport unsigned long GetImageChannelDepth(const Image *image,
+MagickExport size_t GetImageChannelDepth(const Image *image,
   const ChannelType channel,ExceptionInfo *exception)
 {
   CacheView
     *image_view;
 
-  long
+  ssize_t
     y;
 
   MagickBooleanType
     status;
 
-  register long
+  register ssize_t
     id;
 
-  unsigned long
+  size_t
     *current_depth,
     depth,
     number_threads;
@@ -308,26 +307,26 @@ MagickExport unsigned long GetImageChannelDepth(const Image *image,
   if (image->debug != MagickFalse)
     (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",image->filename);
   number_threads=GetOpenMPMaximumThreads();
-  current_depth=(unsigned long *) AcquireQuantumMemory(number_threads,
+  current_depth=(size_t *) AcquireQuantumMemory(number_threads,
     sizeof(*current_depth));
-  if (current_depth == (unsigned long *) NULL)
+  if (current_depth == (size_t *) NULL)
     ThrowFatalException(ResourceLimitFatalError,"MemoryAllocationFailed");
   status=MagickTrue;
-  for (id=0; id < (long) number_threads; id++)
+  for (id=0; id < (ssize_t) number_threads; id++)
     current_depth[id]=1;
   if ((image->storage_class == PseudoClass) && (image->matte == MagickFalse))
     {
       register const PixelPacket
         *restrict p;
 
-      register long
+      register ssize_t
         i;
 
       p=image->colormap;
 #if defined(MAGICKCORE_OPENMP_SUPPORT)
   #pragma omp parallel for schedule(dynamic,4) shared(status)
 #endif
-      for (i=0; i < (long) image->colors; i++)
+      for (i=0; i < (ssize_t) image->colors; i++)
       {
         if (status == MagickFalse)
           continue;
@@ -358,26 +357,28 @@ MagickExport unsigned long GetImageChannelDepth(const Image *image,
         p++;
       }
       depth=current_depth[0];
-      for (id=1; id < (long) number_threads; id++)
+      for (id=1; id < (ssize_t) number_threads; id++)
         if (depth < current_depth[id])
           depth=current_depth[id];
-      current_depth=(unsigned long *) RelinquishMagickMemory(current_depth);
+      current_depth=(size_t *) RelinquishMagickMemory(current_depth);
       return(depth);
     }
   image_view=AcquireCacheView(image);
 #if defined(MAGICKCORE_OPENMP_SUPPORT)
   #pragma omp parallel for schedule(dynamic,4) shared(status)
 #endif
-  for (y=0; y < (long) image->rows; y++)
+  for (y=0; y < (ssize_t) image->rows; y++)
   {
+    int
+      id;
+
     register const IndexPacket
       *restrict indexes;
 
     register const PixelPacket
       *restrict p;
 
-    register long
-      id,
+    register ssize_t
       x;
 
     if (status == MagickFalse)
@@ -387,7 +388,7 @@ MagickExport unsigned long GetImageChannelDepth(const Image *image,
     if (p == (const PixelPacket *) NULL)
       continue;
     indexes=GetCacheViewVirtualIndexQueue(image_view);
-    for (x=0; x < (long) image->columns; x++)
+    for (x=0; x < (ssize_t) image->columns; x++)
     {
       while (current_depth[id] < MAGICKCORE_QUANTUM_DEPTH)
       {
@@ -426,10 +427,10 @@ MagickExport unsigned long GetImageChannelDepth(const Image *image,
   }
   image_view=DestroyCacheView(image_view);
   depth=current_depth[0];
-  for (id=1; id < (long) number_threads; id++)
+  for (id=1; id < (ssize_t) number_threads; id++)
     if (depth < current_depth[id])
       depth=current_depth[id];
-  current_depth=(unsigned long *) RelinquishMagickMemory(current_depth);
+  current_depth=(size_t *) RelinquishMagickMemory(current_depth);
   return(depth);
 }
 
@@ -449,7 +450,7 @@ MagickExport unsigned long GetImageChannelDepth(const Image *image,
 %
 %  The format of the GetImageQuantumDepth method is:
 %
-%      unsigned long GetImageQuantumDepth(const Image *image,
+%      size_t GetImageQuantumDepth(const Image *image,
 %        const MagickBooleanType constrain)
 %
 %  A description of each parameter follows:
@@ -468,10 +469,10 @@ static inline double MagickMin(const double x,const double y)
   return(y);
 }
 
-MagickExport unsigned long GetImageQuantumDepth(const Image *image,
+MagickExport size_t GetImageQuantumDepth(const Image *image,
   const MagickBooleanType constrain)
 {
-  unsigned long
+  size_t
     depth;
 
   depth=image->depth;
@@ -487,7 +488,7 @@ MagickExport unsigned long GetImageQuantumDepth(const Image *image,
         if (depth <= 64)
           depth=64;
   if (constrain != MagickFalse)
-    depth=(unsigned long) MagickMin((double) depth,(double)
+    depth=(size_t) MagickMin((double) depth,(double)
       MAGICKCORE_QUANTUM_DEPTH);
   return(depth);
 }
@@ -584,12 +585,21 @@ MagickExport ImageType GetImageType(const Image *image,ExceptionInfo *exception)
 MagickExport MagickBooleanType IsGrayImage(const Image *image,
   ExceptionInfo *exception)
 {
+  CacheView
+    *image_view;
+
   ImageType
     type;
 
+  ssize_t
+    y;
+
   register const PixelPacket
     *p;
 
+  register ssize_t
+    x;
+
   assert(image != (Image *) NULL);
   assert(image->signature == MagickSignature);
   if (image->debug != MagickFalse)
@@ -600,63 +610,27 @@ MagickExport MagickBooleanType IsGrayImage(const Image *image,
   if (image->colorspace == CMYKColorspace)
     return(MagickFalse);
   type=BilevelType;
-  switch (image->storage_class)
+  image_view=AcquireCacheView(image);
+  for (y=0; y < (ssize_t) image->rows; y++)
   {
-    case DirectClass:
-    case UndefinedClass:
+    p=GetCacheViewVirtualPixels(image_view,0,y,image->columns,1,exception);
+    if (p == (const PixelPacket *) NULL)
+      break;
+    for (x=0; x < (ssize_t) image->columns; x++)
     {
-      long
-        y;
-
-      register long
-        x;
-
-      CacheView
-        *image_view;
-
-      image_view=AcquireCacheView(image);
-      for (y=0; y < (long) image->rows; y++)
-      {
-        p=GetCacheViewVirtualPixels(image_view,0,y,image->columns,1,exception);
-        if (p == (const PixelPacket *) NULL)
-          break;
-        for (x=0; x < (long) image->columns; x++)
+      if (IsGrayPixel(p) == MagickFalse)
         {
-          if (IsGrayPixel(p) == MagickFalse)
-            {
-              type=UndefinedType;
-              break;
-            }
-          if ((type == BilevelType) && (IsMonochromePixel(p) == MagickFalse))
-            type=GrayscaleType;
-          p++;
-        }
-        if (type == UndefinedType)
+          type=UndefinedType;
           break;
-      }
-      image_view=DestroyCacheView(image_view);
-      break;
+        }
+      if ((type == BilevelType) && (IsMonochromePixel(p) == MagickFalse))
+        type=GrayscaleType;
+      p++;
     }
-    case PseudoClass:
-    {
-      register long
-        i;
-
-      p=image->colormap;
-      for (i=0; i < (long) image->colors; i++)
-      {
-        if (IsGrayPixel(p) == MagickFalse)
-          {
-            type=UndefinedType;
-            break;
-          }
-        if ((type == BilevelType) && (IsMonochromePixel(p) == MagickFalse))
-          type=GrayscaleType;
-        p++;
-      }
+    if (type == UndefinedType)
       break;
-    }
   }
+  image_view=DestroyCacheView(image_view);
   if (type == UndefinedType)
     return(MagickFalse);
   ((Image *) image)->type=type;
@@ -695,9 +669,18 @@ MagickExport MagickBooleanType IsGrayImage(const Image *image,
 MagickExport MagickBooleanType IsMonochromeImage(const Image *image,
   ExceptionInfo *exception)
 {
+  CacheView
+    *image_view;
+
   ImageType
     type;
 
+  ssize_t
+    y;
+
+  register ssize_t
+    x;
+
   register const PixelPacket
     *p;
 
@@ -710,61 +693,25 @@ MagickExport MagickBooleanType IsMonochromeImage(const Image *image,
   if (image->colorspace == CMYKColorspace)
     return(MagickFalse);
   type=BilevelType;
-  switch (image->storage_class)
+  image_view=AcquireCacheView(image);
+  for (y=0; y < (ssize_t) image->rows; y++)
   {
-    case DirectClass:
-    case UndefinedClass:
+    p=GetCacheViewVirtualPixels(image_view,0,y,image->columns,1,exception);
+    if (p == (const PixelPacket *) NULL)
+      break;
+    for (x=0; x < (ssize_t) image->columns; x++)
     {
-      long
-        y;
-
-      register long
-        x;
-
-      CacheView
-        *image_view;
-
-      image_view=AcquireCacheView(image);
-      for (y=0; y < (long) image->rows; y++)
-      {
-        p=GetCacheViewVirtualPixels(image_view,0,y,image->columns,1,exception);
-        if (p == (const PixelPacket *) NULL)
-          break;
-        for (x=0; x < (long) image->columns; x++)
+      if (IsMonochromePixel(p) == MagickFalse)
         {
-          if (IsMonochromePixel(p) == MagickFalse)
-            {
-              type=UndefinedType;
-              break;
-            }
-          p++;
-        }
-        if (type == UndefinedType)
+          type=UndefinedType;
           break;
-      }
-      image_view=DestroyCacheView(image_view);
-      if (y == (long) image->rows)
-        ((Image *) image)->type=BilevelType;
-      break;
+        }
+      p++;
     }
-    case PseudoClass:
-    {
-      register long
-        i;
-
-      p=image->colormap;
-      for (i=0; i < (long) image->colors; i++)
-      {
-        if (IsMonochromePixel(p) == MagickFalse)
-          {
-            type=UndefinedType;
-            break;
-          }
-        p++;
-      }
+    if (type == UndefinedType)
       break;
-    }
   }
+  image_view=DestroyCacheView(image_view);
   if (type == UndefinedType)
     return(MagickFalse);
   ((Image *) image)->type=type;
@@ -803,13 +750,13 @@ MagickExport MagickBooleanType IsOpaqueImage(const Image *image,
   CacheView
     *image_view;
 
-  long
+  ssize_t
     y;
 
   register const PixelPacket
     *p;
 
-  register long
+  register ssize_t
     x;
 
   /*
@@ -822,22 +769,22 @@ MagickExport MagickBooleanType IsOpaqueImage(const Image *image,
   if (image->matte == MagickFalse)
     return(MagickTrue);
   image_view=AcquireCacheView(image);
-  for (y=0; y < (long) image->rows; y++)
+  for (y=0; y < (ssize_t) image->rows; y++)
   {
     p=GetCacheViewVirtualPixels(image_view,0,y,image->columns,1,exception);
     if (p == (const PixelPacket *) NULL)
       break;
-    for (x=0; x < (long) image->columns; x++)
+    for (x=0; x < (ssize_t) image->columns; x++)
     {
       if (p->opacity != OpaqueOpacity)
         break;
       p++;
     }
-    if (x < (long) image->columns)
+    if (x < (ssize_t) image->columns)
      break;
   }
   image_view=DestroyCacheView(image_view);
-  return(y < (long) image->rows ? MagickFalse : MagickTrue);
+  return(y < (ssize_t) image->rows ? MagickFalse : MagickTrue);
 }
 
 /*
@@ -855,9 +802,9 @@ MagickExport MagickBooleanType IsOpaqueImage(const Image *image,
 %
 %  The format of the SetImageChannelDepth method is:
 %
-%      MagickBooleanType SetImageDepth(Image *image,const unsigned long depth)
+%      MagickBooleanType SetImageDepth(Image *image,const size_t depth)
 %      MagickBooleanType SetImageChannelDepth(Image *image,
-%        const ChannelType channel,const unsigned long depth)
+%        const ChannelType channel,const size_t depth)
 %
 %  A description of each parameter follows:
 %
@@ -868,15 +815,14 @@ MagickExport MagickBooleanType IsOpaqueImage(const Image *image,
 %    o depth: the image depth.
 %
 */
-
 MagickExport MagickBooleanType SetImageDepth(Image *image,
-  const unsigned long depth)
+  const size_t depth)
 {
   return(SetImageChannelDepth(image,AllChannels,depth));
 }
 
 MagickExport MagickBooleanType SetImageChannelDepth(Image *image,
-  const ChannelType channel,const unsigned long depth)
+  const ChannelType channel,const size_t depth)
 {
   CacheView
     *image_view;
@@ -884,7 +830,7 @@ MagickExport MagickBooleanType SetImageChannelDepth(Image *image,
   ExceptionInfo
     *exception;
 
-  long
+  ssize_t
     y;
 
   MagickBooleanType
@@ -897,7 +843,7 @@ MagickExport MagickBooleanType SetImageChannelDepth(Image *image,
   if (image->debug != MagickFalse)
     (void) LogMagickEvent(TraceEvent,GetMagickModule(),"...");
   assert(image->signature == MagickSignature);
-  if (GetImageDepth(image,&image->exception) <= (unsigned long)
+  if (GetImageDepth(image,&image->exception) <= (size_t)
       MagickMin((double) depth,(double) MAGICKCORE_QUANTUM_DEPTH))
     {
       image->depth=depth;
@@ -913,12 +859,12 @@ MagickExport MagickBooleanType SetImageChannelDepth(Image *image,
 #if defined(MAGICKCORE_OPENMP_SUPPORT)
   #pragma omp parallel for schedule(dynamic,4) shared(status)
 #endif
-  for (y=0; y < (long) image->rows; y++)
+  for (y=0; y < (ssize_t) image->rows; y++)
   {
     register IndexPacket
       *restrict indexes;
 
-    register long
+    register ssize_t
       x;
 
     register PixelPacket
@@ -934,7 +880,7 @@ MagickExport MagickBooleanType SetImageChannelDepth(Image *image,
         continue;
       }
     indexes=GetCacheViewAuthenticIndexQueue(image_view);
-    for (x=0; x < (long) image->columns; x++)
+    for (x=0; x < (ssize_t) image->columns; x++)
     {
       if ((channel & RedChannel) != 0)
         q->red=ScaleAnyToQuantum(ScaleQuantumToAny(q->red,range),range);
@@ -958,21 +904,17 @@ MagickExport MagickBooleanType SetImageChannelDepth(Image *image,
   image_view=DestroyCacheView(image_view);
   if (image->storage_class == PseudoClass)
     {
-      QuantumAny
-        range;
-
-      register long
+      register ssize_t
         i;
 
       register PixelPacket
         *restrict p;
 
       p=image->colormap;
-      range=GetQuantumRange(depth);
 #if defined(MAGICKCORE_OPENMP_SUPPORT)
   #pragma omp parallel for schedule(dynamic,4) shared(status)
 #endif
-      for (i=0; i < (long) image->colors; i++)
+      for (i=0; i < (ssize_t) image->colors; i++)
       {
         if ((channel & RedChannel) != 0)
           p->red=ScaleAnyToQuantum(ScaleQuantumToAny(p->red,range),range);
