@@ -303,12 +303,12 @@ MagickExport ExceptionInfo *GetCacheViewException(const CacheView *cache_view)
 */
 MagickExport MagickSizeType GetCacheViewExtent(const CacheView *cache_view)
 {
-  ssize_t
-    id;
-
   MagickSizeType
     extent;
 
+  ssize_t
+    id;
+
   assert(cache_view != (CacheView *) NULL);
   assert(cache_view->signature == MagickSignature);
   if (cache_view->debug != MagickFalse)
@@ -385,23 +385,20 @@ MagickExport ClassType GetCacheViewStorageClass(const CacheView *cache_view)
 %
 */
 MagickExport PixelPacket *GetCacheViewAuthenticPixels(CacheView *cache_view,
-  const ssize_t x,const ssize_t y,const size_t columns,
-  const size_t rows,ExceptionInfo *exception)
+  const ssize_t x,const ssize_t y,const size_t columns,const size_t rows,
+  ExceptionInfo *exception)
 {
   Cache
     cache;
 
-  ssize_t
-    id;
-
   PixelPacket
     *pixels;
 
+  ssize_t
+    id;
+
   assert(cache_view != (CacheView *) NULL);
   assert(cache_view->signature == MagickSignature);
-  if (cache_view->debug != MagickFalse)
-    (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",
-      cache_view->image->filename);
   cache=GetImagePixelCache(cache_view->image,MagickTrue,exception);
   if (cache == (Cache) NULL)
     return((PixelPacket *) NULL);
@@ -444,23 +441,20 @@ MagickExport PixelPacket *GetCacheViewAuthenticPixels(CacheView *cache_view,
 %
 */
 MagickExport MagickBooleanType GetOneCacheViewAuthenticPixel(
-  const CacheView *cache_view,const ssize_t x,const ssize_t y,PixelPacket *pixel,
-  ExceptionInfo *exception)
+  const CacheView *cache_view,const ssize_t x,const ssize_t y,
+  PixelPacket *pixel,ExceptionInfo *exception)
 {
   Cache
     cache;
 
-  ssize_t
-    id;
-
   PixelPacket
     *pixels;
 
+  ssize_t
+    id;
+
   assert(cache_view != (CacheView *) NULL);
   assert(cache_view->signature == MagickSignature);
-  if (cache_view->debug != MagickFalse)
-    (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",
-      cache_view->image->filename);
   cache=GetImagePixelCache(cache_view->image,MagickTrue,exception);
   if (cache == (Cache) NULL)
     return(MagickFalse);
@@ -509,9 +503,6 @@ MagickExport IndexPacket *GetCacheViewAuthenticIndexQueue(CacheView *cache_view)
 
   assert(cache_view != (CacheView *) NULL);
   assert(cache_view->signature == MagickSignature);
-  if (cache_view->debug != MagickFalse)
-    (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",
-      cache_view->image->filename);
   assert(cache_view->image->cache != (Cache) NULL);
   id=GetOpenMPThreadId();
   assert(id < (ssize_t) cache_view->number_threads);
@@ -547,17 +538,14 @@ MagickExport IndexPacket *GetCacheViewAuthenticIndexQueue(CacheView *cache_view)
 */
 MagickExport PixelPacket *GetCacheViewAuthenticPixelQueue(CacheView *cache_view)
 {
-  ssize_t
-    id;
-
   PixelPacket
     *pixels;
 
+  ssize_t
+    id;
+
   assert(cache_view != (CacheView *) NULL);
   assert(cache_view->signature == MagickSignature);
-  if (cache_view->debug != MagickFalse)
-    (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",
-      cache_view->image->filename);
   assert(cache_view->image->cache != (Cache) NULL);
   id=GetOpenMPThreadId();
   assert(id < (ssize_t) cache_view->number_threads);
@@ -602,9 +590,6 @@ MagickExport const IndexPacket *GetCacheViewVirtualIndexQueue(
 
   assert(cache_view != (const CacheView *) NULL);
   assert(cache_view->signature == MagickSignature);
-  if (cache_view->debug != MagickFalse)
-    (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",
-      cache_view->image->filename);
   assert(cache_view->image->cache != (Cache) NULL);
   id=GetOpenMPThreadId();
   assert(id < (ssize_t) cache_view->number_threads);
@@ -649,9 +634,6 @@ MagickExport const PixelPacket *GetCacheViewVirtualPixelQueue(
 
   assert(cache_view != (const CacheView *) NULL);
   assert(cache_view->signature == MagickSignature);
-  if (cache_view->debug != MagickFalse)
-    (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",
-      cache_view->image->filename);
   assert(cache_view->image->cache != (Cache) NULL);
   id=GetOpenMPThreadId();
   assert(id < (ssize_t) cache_view->number_threads);
@@ -680,8 +662,7 @@ MagickExport const PixelPacket *GetCacheViewVirtualPixelQueue(
 %
 %      const PixelPacket *GetCacheViewVirtualPixels(
 %        const CacheView *cache_view,const ssize_t x,const ssize_t y,
-%        const size_t columns,const size_t rows,
-%        ExceptionInfo *exception)
+%        const size_t columns,const size_t rows,ExceptionInfo *exception)
 %
 %  A description of each parameter follows:
 %
@@ -705,9 +686,6 @@ MagickExport const PixelPacket *GetCacheViewVirtualPixels(
 
   assert(cache_view != (CacheView *) NULL);
   assert(cache_view->signature == MagickSignature);
-  if (cache_view->debug != MagickFalse)
-    (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",
-      cache_view->image->filename);
   id=GetOpenMPThreadId();
   assert(id < (ssize_t) cache_view->number_threads);
   pixels=GetVirtualPixelsFromNexus(cache_view->image,
@@ -749,8 +727,8 @@ MagickExport const PixelPacket *GetCacheViewVirtualPixels(
 %
 */
 MagickExport MagickBooleanType GetOneCacheViewVirtualPixel(
-  const CacheView *cache_view,const ssize_t x,const ssize_t y,PixelPacket *pixel,
-  ExceptionInfo *exception)
+  const CacheView *cache_view,const ssize_t x,const ssize_t y,
+  PixelPacket *pixel,ExceptionInfo *exception)
 {
   const PixelPacket
     *pixels;
@@ -760,9 +738,6 @@ MagickExport MagickBooleanType GetOneCacheViewVirtualPixel(
 
   assert(cache_view != (CacheView *) NULL);
   assert(cache_view->signature == MagickSignature);
-  if (cache_view->debug != MagickFalse)
-    (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",
-      cache_view->image->filename);
   *pixel=cache_view->image->background_color;
   id=GetOpenMPThreadId();
   assert(id < (ssize_t) cache_view->number_threads);
@@ -823,9 +798,6 @@ MagickExport MagickBooleanType GetOneCacheViewVirtualMethodPixel(
 
   assert(cache_view != (CacheView *) NULL);
   assert(cache_view->signature == MagickSignature);
-  if (cache_view->debug != MagickFalse)
-    (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",
-      cache_view->image->filename);
   *pixel=cache_view->image->background_color;
   id=GetOpenMPThreadId();
   assert(id < (ssize_t) cache_view->number_threads);
@@ -870,23 +842,20 @@ MagickExport MagickBooleanType GetOneCacheViewVirtualMethodPixel(
 %
 */
 MagickExport PixelPacket *QueueCacheViewAuthenticPixels(CacheView *cache_view,
-  const ssize_t x,const ssize_t y,const size_t columns,
-  const size_t rows,ExceptionInfo *exception)
+  const ssize_t x,const ssize_t y,const size_t columns,const size_t rows,
+  ExceptionInfo *exception)
 {
   Cache
     cache;
 
-  ssize_t
-    id;
-
   PixelPacket
     *pixels;
 
+  ssize_t
+    id;
+
   assert(cache_view != (CacheView *) NULL);
   assert(cache_view->signature == MagickSignature);
-  if (cache_view->debug != MagickFalse)
-    (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",
-      cache_view->image->filename);
   cache=GetImagePixelCache(cache_view->image,MagickFalse,exception);
   if (cache == (Cache) NULL)
     return((PixelPacket *) NULL);
@@ -1002,17 +971,14 @@ MagickExport MagickBooleanType SetCacheViewVirtualPixelMethod(
 MagickExport MagickBooleanType SyncCacheViewAuthenticPixels(
   CacheView *cache_view,ExceptionInfo *exception)
 {
-  ssize_t
-    id;
-
   MagickBooleanType
     status;
 
+  ssize_t
+    id;
+
   assert(cache_view != (CacheView *) NULL);
   assert(cache_view->signature == MagickSignature);
-  if (cache_view->debug != MagickFalse)
-    (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",
-      cache_view->image->filename);
   id=GetOpenMPThreadId();
   assert(id < (ssize_t) cache_view->number_threads);
   status=SyncAuthenticPixelCacheNexus(cache_view->image,
