@@ -626,6 +626,9 @@ MagickExport Image *ReadImage(const ImageInfo *image_info,
     char
       timestamp[MaxTextExtent];
 
+    const char
+      *option;
+
     const StringInfo
       *profile;
 
@@ -771,8 +774,36 @@ MagickExport Image *ReadImage(const ImageInfo *image_info,
     (void) FormatMagickTime(GetBlobProperties(next)->st_ctime,MaxTextExtent,
       timestamp);
     (void) SetImageProperty(next,"date:create",timestamp);
+    option=GetImageOption(image_info,"delay");
+    if (option != (const char *) NULL)
+      {
+        GeometryInfo
+          geometry_info;
+
+        flags=ParseGeometry(option,&geometry_info);
+        if ((flags & GreaterValue) != 0)
+          {
+            if (image->delay > (size_t) floor(geometry_info.rho+0.5))
+              image->delay=(size_t) floor(geometry_info.rho+0.5);
+          }
+        else
+          if ((flags & LessValue) != 0)
+            {
+              if (image->delay < (size_t) floor(geometry_info.rho+0.5))
+                image->ticks_per_second=(ssize_t) floor(geometry_info.sigma+
+                  0.5);
+            }
+          else
+            image->delay=(size_t) floor(geometry_info.rho+0.5);
+        if ((flags & SigmaValue) != 0)
+          image->ticks_per_second=(ssize_t) floor(geometry_info.sigma+0.5);
+      }
+    option=GetImageOption(image_info,"dispose");
+    if (option != (const char *) NULL)
+      image->dispose=(DisposeType) ParseMagickOption(MagickDisposeOptions,
+        MagickFalse,option);
     if (read_info->verbose != MagickFalse)
-      (void) IdentifyImage(next,stdout,MagickFalse);
+      (void) IdentifyImage(next,stderr,MagickFalse);
     image=next;
   }
   read_info=DestroyImageInfo(read_info);
@@ -1093,6 +1124,7 @@ MagickExport MagickBooleanType WriteImage(const ImageInfo *image_info,
               /*
                 A seekable stream is required by the encoder.
               */
+              write_info->adjoin=MagickTrue;
               (void) CopyMagickString(write_info->filename,image->filename,
                 MaxTextExtent);
               (void) AcquireUniqueFilename(image->filename);
@@ -1182,10 +1214,10 @@ MagickExport MagickBooleanType WriteImage(const ImageInfo *image_info,
       status=OpenBlob(write_info,image,ReadBinaryBlobMode,&image->exception);
       if (status != MagickFalse)
         status=ImageToFile(image,write_info->filename,&image->exception);
+      (void) CloseBlob(image);
       (void) RelinquishUniqueFileResource(image->filename);
       (void) CopyMagickString(image->filename,write_info->filename,
         MaxTextExtent);
-      (void) CloseBlob(image);
     }
   if ((LocaleCompare(write_info->magick,"info") != 0) &&
       (write_info->verbose != MagickFalse))
