@@ -70,7 +70,7 @@
 #define BlockHeader(block)  ((size_t *) (block)-1)
 #define BlockSize  4096
 #define BlockThreshold  1024
-#define AlignedSize  (32*sizeof(void *))
+#define AlignedSize  (16*sizeof(void *))
 #define MaxBlockExponent  16
 #define MaxBlocks ((BlockThreshold/(4*sizeof(size_t)))+MaxBlockExponent+1)
 #define MaxSegments  1024
@@ -186,6 +186,14 @@ static MagickBooleanType
 %    o quantum: the number of bytes in each quantum.
 %
 */
+
+static inline size_t MagickMax(const size_t x,const size_t y)
+{
+  if (x > y)
+    return(x);
+  return(y);
+}
+
 MagickExport void *AcquireAlignedMemory(const size_t count,const size_t quantum)
 {
   size_t
@@ -202,7 +210,7 @@ MagickExport void *AcquireAlignedMemory(const size_t count,const size_t quantum)
     void
       *memory;
 
-    if (posix_memalign(&memory,AlignedSize,size) == 0)
+    if (posix_memalign(&memory,AlignedSize,MagickMax(size,AlignedSize)) == 0)
       return(memory);
   }
 #endif
@@ -396,7 +404,7 @@ MagickExport void *AcquireMagickMemory(const size_t size)
       LockSemaphoreInfo(memory_semaphore);
       if (free_segments == (DataSegmentInfo *) NULL)
         {
-          register long
+          register ssize_t
             i;
 
           assert(2*sizeof(size_t) > (size_t) (~SizeMask));
@@ -512,6 +520,7 @@ MagickExport void *CopyMagickMemory(void *destination,const void *source,
     switch (size)
     {
       default: return(memcpy(destination,source,size));
+      case 8: *q++=(*p++);
       case 7: *q++=(*p++);
       case 6: *q++=(*p++);
       case 5: *q++=(*p++);
@@ -545,14 +554,14 @@ MagickExport void *CopyMagickMemory(void *destination,const void *source,
 MagickExport void DestroyMagickMemory(void)
 {
 #if defined(MAGICKCORE_EMBEDDABLE_SUPPORT)
-  register long
+  register ssize_t
     i;
 
   if (memory_semaphore == (SemaphoreInfo *) NULL)
     AcquireSemaphoreInfo(&memory_semaphore);
   LockSemaphoreInfo(memory_semaphore);
   UnlockSemaphoreInfo(memory_semaphore);
-  for (i=0; i < (long) memory_info.number_segments; i++)
+  for (i=0; i < (ssize_t) memory_info.number_segments; i++)
     if (memory_info.segments[i]->mapped == MagickFalse)
       memory_methods.destroy_memory_handler(
         memory_info.segments[i]->allocation);
@@ -597,7 +606,7 @@ static MagickBooleanType ExpandHeap(size_t size)
   MagickBooleanType
     mapped;
 
-  register long
+  register ssize_t
     i;
 
   register void
@@ -623,7 +632,7 @@ static MagickBooleanType ExpandHeap(size_t size)
   segment_info->length=blocksize;
   segment_info->allocation=segment;
   segment_info->bound=(char *) segment+blocksize;
-  i=(long) memory_info.number_segments-1;
+  i=(ssize_t) memory_info.number_segments-1;
   for ( ; (i >= 0) && (memory_info.segments[i]->allocation > segment); i--)
     memory_info.segments[i+1]=memory_info.segments[i];
   memory_info.segments[i+1]=segment_info;
