@@ -29,6 +29,9 @@ extern "C" {
 #define WIN32_LEAN_AND_MEAN
 #define VC_EXTRALEAN
 #define _CRT_SECURE_NO_DEPRECATE  1
+#if !defined(_WIN32_WINNT)
+#define _WIN32_WINNT 0x0501
+#endif
 #include <windows.h>
 #include <wchar.h>
 #include <winuser.h>
@@ -96,7 +99,7 @@ extern "C" {
 #  define fseeko  _fseeki64
 #endif
 #if !defined(fstat) && !defined(__BORLANDC__)
-#if defined(__WINDOWS__) && !defined(Windows95) && \
+#if defined(MAGICKCORE_WINDOWS_SUPPORT) && !defined(Windows95) && \
   !(defined(_MSC_VER) && (_MSC_VER < 1400)) &&  (__MSVCRT_VERSION__ < 0x800)
 #  define fstat  _fstati64
 #else
@@ -191,7 +194,7 @@ extern "C" {
 #  define spawnvp  _spawnvp
 #endif
 #if !defined(stat) && !defined(__BORLANDC__)
-#if defined(__WINDOWS__) && !defined(Windows95) && \
+#if defined(MAGICKCORE_WINDOWS_SUPPORT) && !defined(Windows95) && \
   !(defined(_MSC_VER) && (_MSC_VER < 1400)) &&  (__MSVCRT_VERSION__ < 0x800)
 #  define stat  _stati64
 #else
@@ -222,7 +225,7 @@ extern "C" {
 #  define write  _write
 #endif
 #if !defined(wstat) && !defined(__BORLANDC__)
-#if defined(__WINDOWS__) && !defined(Windows95) && \
+#if defined(MAGICKCORE_WINDOWS_SUPPORT) && !defined(Windows95) && \
   !(defined(_MSC_VER) && (_MSC_VER < 1400)) &&  (__MSVCRT_VERSION__ < 0x800)
 #  define wstat  _wstati64
 #else
@@ -230,7 +233,7 @@ extern "C" {
 #endif
 #endif
 
-#if defined(_MT) && defined(__WINDOWS__)
+#if defined(_MT) && defined(MAGICKCORE_WINDOWS_SUPPORT)
 #  define SAFE_GLOBAL  __declspec(thread)
 #else
 #  define SAFE_GLOBAL
@@ -288,6 +291,15 @@ typedef struct _NTMEMORYSTATUSEX
     ullAvailExtendedVirtual;
 } NTMEMORYSTATUSEX;
 
+#if !defined(__MINGW32__)
+struct timezone
+{
+  int
+    tz_minuteswest,
+    tz_dsttime;
+};
+#endif
+
 typedef UINT
   (CALLBACK *LPFNDLLFUNC1)(DWORD,UINT);
 
@@ -296,10 +308,6 @@ typedef UINT
 
 #endif
 
-#if !defined(ssize_t) && !defined(__MINGW32__)
-typedef long ssize_t;
-#endif
-
 #if defined(MAGICKCORE_BZLIB_DELEGATE)
 #  if defined(_WIN32)
 #    define BZ_IMPORT 1
@@ -330,6 +338,9 @@ extern MagickExport double
 
 extern MagickExport int
   Exit(int),
+#if !defined(__MINGW32__)
+  gettimeofday(struct timeval *,struct timezone *),
+#endif
   IsWindows95(),
   NTCloseDirectory(DIR *),
   NTCloseLibrary(void *),
@@ -347,7 +358,7 @@ extern MagickExport int
   NTUnmapMemory(void *,size_t),
   NTSystemCommand(const char *);
 
-extern MagickExport long
+extern MagickExport ssize_t
   NTSystemConfiguration(int),
   NTTellDirectory(DIR *);
 
@@ -370,7 +381,7 @@ extern MagickExport void
   *NTGetLibrarySymbol(void *,const char *),
   *NTMapMemory(char *,size_t,int,int,int,MagickOffsetType),
   *NTOpenLibrary(const char *),
-  NTSeekDirectory(DIR *,long),
+  NTSeekDirectory(DIR *,ssize_t),
   NTWarningHandler(const ExceptionType,const char *,const char *);
 
 #endif /* !XS_VERSION */
