@@ -105,7 +105,7 @@ MagickExport Image *ChopImage(const Image *image,const RectangleInfo *chop_info,
   Image
     *chop_image;
 
-  long
+  ssize_t
     j,
     y;
 
@@ -116,7 +116,7 @@ MagickExport Image *ChopImage(const Image *image,const RectangleInfo *chop_info,
   RectangleInfo
     extent;
 
-  register long
+  register ssize_t
     i;
 
   /*
@@ -129,24 +129,24 @@ MagickExport Image *ChopImage(const Image *image,const RectangleInfo *chop_info,
   assert(exception != (ExceptionInfo *) NULL);
   assert(exception->signature == MagickSignature);
   assert(chop_info != (RectangleInfo *) NULL);
-  if (((chop_info->x+(long) chop_info->width) < 0) ||
-      ((chop_info->y+(long) chop_info->height) < 0) ||
-      (chop_info->x > (long) image->columns) ||
-      (chop_info->y > (long) image->rows))
+  if (((chop_info->x+(ssize_t) chop_info->width) < 0) ||
+      ((chop_info->y+(ssize_t) chop_info->height) < 0) ||
+      (chop_info->x > (ssize_t) image->columns) ||
+      (chop_info->y > (ssize_t) image->rows))
     ThrowImageException(OptionWarning,"GeometryDoesNotContainImage");
   extent=(*chop_info);
-  if ((extent.x+(long) extent.width) > (long) image->columns)
-    extent.width=(unsigned long) ((long) image->columns-extent.x);
-  if ((extent.y+(long) extent.height) > (long) image->rows)
-    extent.height=(unsigned long) ((long) image->rows-extent.y);
+  if ((extent.x+(ssize_t) extent.width) > (ssize_t) image->columns)
+    extent.width=(size_t) ((ssize_t) image->columns-extent.x);
+  if ((extent.y+(ssize_t) extent.height) > (ssize_t) image->rows)
+    extent.height=(size_t) ((ssize_t) image->rows-extent.y);
   if (extent.x < 0)
     {
-      extent.width-=(unsigned long) (-extent.x);
+      extent.width-=(size_t) (-extent.x);
       extent.x=0;
     }
   if (extent.y < 0)
     {
-      extent.height-=(unsigned long) (-extent.y);
+      extent.height-=(size_t) (-extent.y);
       extent.y=0;
     }
   chop_image=CloneImage(image,image->columns-extent.width,image->rows-
@@ -161,7 +161,7 @@ MagickExport Image *ChopImage(const Image *image,const RectangleInfo *chop_info,
   j=0;
   image_view=AcquireCacheView(image);
   chop_view=AcquireCacheView(chop_image);
-  for (y=0; y < (long) extent.y; y++)
+  for (y=0; y < (ssize_t) extent.y; y++)
   {
     register const PixelPacket
       *restrict p;
@@ -170,7 +170,7 @@ MagickExport Image *ChopImage(const Image *image,const RectangleInfo *chop_info,
       *restrict chop_indexes,
       *restrict indexes;
 
-    register long
+    register ssize_t
       x;
 
     register PixelPacket
@@ -188,9 +188,9 @@ MagickExport Image *ChopImage(const Image *image,const RectangleInfo *chop_info,
       }
     indexes=GetCacheViewAuthenticIndexQueue(image_view);
     chop_indexes=GetCacheViewAuthenticIndexQueue(chop_view);
-    for (x=0; x < (long) image->columns; x++)
+    for (x=0; x < (ssize_t) image->columns; x++)
     {
-      if ((x < extent.x) || (x >= (long) (extent.x+extent.width)))
+      if ((x < extent.x) || (x >= (ssize_t) (extent.x+extent.width)))
         {
           *q=(*p);
           if (indexes != (IndexPacket *) NULL)
@@ -204,15 +204,16 @@ MagickExport Image *ChopImage(const Image *image,const RectangleInfo *chop_info,
     }
     if (SyncCacheViewAuthenticPixels(chop_view,exception) == MagickFalse)
       status=MagickFalse;
-    proceed=SetImageProgress(image,ChopImageTag,y,chop_image->rows);
+    proceed=SetImageProgress(image,ChopImageTag,(MagickOffsetType) y,
+      chop_image->rows);
     if (proceed == MagickFalse)
       status=MagickFalse;
   }
   /*
     Extract chop image.
   */
-  i+=extent.height;
-  for (y=0; y < (long) (image->rows-(extent.y+extent.height)); y++)
+  i+=(ssize_t) extent.height;
+  for (y=0; y < (ssize_t) (image->rows-(extent.y+extent.height)); y++)
   {
     register const PixelPacket
       *restrict p;
@@ -221,7 +222,7 @@ MagickExport Image *ChopImage(const Image *image,const RectangleInfo *chop_info,
       *restrict chop_indexes,
       *restrict indexes;
 
-    register long
+    register ssize_t
       x;
 
     register PixelPacket
@@ -239,9 +240,9 @@ MagickExport Image *ChopImage(const Image *image,const RectangleInfo *chop_info,
       }
     indexes=GetCacheViewAuthenticIndexQueue(image_view);
     chop_indexes=GetCacheViewAuthenticIndexQueue(chop_view);
-    for (x=0; x < (long) image->columns; x++)
+    for (x=0; x < (ssize_t) image->columns; x++)
     {
-      if ((x < extent.x) || (x >= (long) (extent.x+extent.width)))
+      if ((x < extent.x) || (x >= (ssize_t) (extent.x+extent.width)))
         {
           *q=(*p);
           if (indexes != (IndexPacket *) NULL)
@@ -255,7 +256,8 @@ MagickExport Image *ChopImage(const Image *image,const RectangleInfo *chop_info,
     }
     if (SyncCacheViewAuthenticPixels(chop_view,exception) == MagickFalse)
       status=MagickFalse;
-    proceed=SetImageProgress(image,ChopImageTag,y,chop_image->rows);
+    proceed=SetImageProgress(image,ChopImageTag,(MagickOffsetType) y,
+      chop_image->rows);
     if (proceed == MagickFalse)
       status=MagickFalse;
   }
@@ -297,10 +299,10 @@ MagickExport Image *ConsolidateCMYKImages(const Image *images,
     *cmyk_image,
     *cmyk_images;
 
-  long
+  ssize_t
     y;
 
-  register long
+  register ssize_t
     i;
 
   /*
@@ -313,7 +315,7 @@ MagickExport Image *ConsolidateCMYKImages(const Image *images,
   assert(exception != (ExceptionInfo *) NULL);
   assert(exception->signature == MagickSignature);
   cmyk_images=NewImageList();
-  for (i=0; i < (long) GetImageListLength(images); i+=4)
+  for (i=0; i < (ssize_t) GetImageListLength(images); i+=4)
   {
     cmyk_image=CloneImage(images,images->columns,images->rows,MagickTrue,
       exception);
@@ -322,12 +324,12 @@ MagickExport Image *ConsolidateCMYKImages(const Image *images,
     if (SetImageStorageClass(cmyk_image,DirectClass) == MagickFalse)
       break;
     (void) SetImageColorspace(cmyk_image,CMYKColorspace);
-    for (y=0; y < (long) images->rows; y++)
+    for (y=0; y < (ssize_t) images->rows; y++)
     {
       register const PixelPacket
         *restrict p;
 
-      register long
+      register ssize_t
         x;
 
       register PixelPacket
@@ -337,7 +339,7 @@ MagickExport Image *ConsolidateCMYKImages(const Image *images,
       q=QueueAuthenticPixels(cmyk_image,0,y,cmyk_image->columns,1,exception);
       if ((p == (const PixelPacket *) NULL) || (q == (PixelPacket *) NULL))
         break;
-      for (x=0; x < (long) images->columns; x++)
+      for (x=0; x < (ssize_t) images->columns; x++)
       {
         q->red=(Quantum) (QuantumRange-PixelIntensityToQuantum(p));
         p++;
@@ -349,12 +351,12 @@ MagickExport Image *ConsolidateCMYKImages(const Image *images,
     images=GetNextImageInList(images);
     if (images == (Image *) NULL)
       break;
-    for (y=0; y < (long) images->rows; y++)
+    for (y=0; y < (ssize_t) images->rows; y++)
     {
       register const PixelPacket
         *restrict p;
 
-      register long
+      register ssize_t
         x;
 
       register PixelPacket
@@ -364,7 +366,7 @@ MagickExport Image *ConsolidateCMYKImages(const Image *images,
       q=GetAuthenticPixels(cmyk_image,0,y,cmyk_image->columns,1,exception);
       if ((p == (const PixelPacket *) NULL) || (q == (PixelPacket *) NULL))
         break;
-      for (x=0; x < (long) images->columns; x++)
+      for (x=0; x < (ssize_t) images->columns; x++)
       {
         q->green=(Quantum) (QuantumRange-PixelIntensityToQuantum(p));
         p++;
@@ -376,12 +378,12 @@ MagickExport Image *ConsolidateCMYKImages(const Image *images,
     images=GetNextImageInList(images);
     if (images == (Image *) NULL)
       break;
-    for (y=0; y < (long) images->rows; y++)
+    for (y=0; y < (ssize_t) images->rows; y++)
     {
       register const PixelPacket
         *restrict p;
 
-      register long
+      register ssize_t
         x;
 
       register PixelPacket
@@ -391,7 +393,7 @@ MagickExport Image *ConsolidateCMYKImages(const Image *images,
       q=GetAuthenticPixels(cmyk_image,0,y,cmyk_image->columns,1,exception);
       if ((p == (const PixelPacket *) NULL) || (q == (PixelPacket *) NULL))
         break;
-      for (x=0; x < (long) images->columns; x++)
+      for (x=0; x < (ssize_t) images->columns; x++)
       {
         q->blue=(Quantum) (QuantumRange-PixelIntensityToQuantum(p));
         p++;
@@ -403,7 +405,7 @@ MagickExport Image *ConsolidateCMYKImages(const Image *images,
     images=GetNextImageInList(images);
     if (images == (Image *) NULL)
       break;
-    for (y=0; y < (long) images->rows; y++)
+    for (y=0; y < (ssize_t) images->rows; y++)
     {
       register const PixelPacket
         *restrict p;
@@ -411,7 +413,7 @@ MagickExport Image *ConsolidateCMYKImages(const Image *images,
       register IndexPacket
         *restrict indexes;
 
-      register long
+      register ssize_t
         x;
 
       register PixelPacket
@@ -422,7 +424,7 @@ MagickExport Image *ConsolidateCMYKImages(const Image *images,
       if ((p == (const PixelPacket *) NULL) || (q == (PixelPacket *) NULL))
         break;
       indexes=GetAuthenticIndexQueue(cmyk_image);
-      for (x=0; x < (long) images->columns; x++)
+      for (x=0; x < (ssize_t) images->columns; x++)
       {
         indexes[x]=(IndexPacket) (QuantumRange-PixelIntensityToQuantum(p));
         p++;
@@ -479,17 +481,19 @@ MagickExport Image *CropImage(const Image *image,const RectangleInfo *geometry,
   Image
     *crop_image;
 
-  long
-    progress,
-    y;
-
   MagickBooleanType
     status;
 
+  MagickOffsetType
+    progress;
+
   RectangleInfo
     bounding_box,
     page;
 
+  ssize_t
+    y;
+
   /*
     Check crop geometry.
   */
@@ -511,10 +515,10 @@ MagickExport Image *CropImage(const Image *image,const RectangleInfo *geometry,
     page.width=bounding_box.width;
   if (page.height == 0)
     page.height=bounding_box.height;
-  if (((bounding_box.x-page.x) >= (long) page.width) ||
-      ((bounding_box.y-page.y) >= (long) page.height) ||
-      ((page.x-bounding_box.x) > (long) image->columns) ||
-      ((page.y-bounding_box.y) > (long) image->rows))
+  if (((bounding_box.x-page.x) >= (ssize_t) page.width) ||
+      ((bounding_box.y-page.y) >= (ssize_t) page.height) ||
+      ((page.x-bounding_box.x) > (ssize_t) image->columns) ||
+      ((page.y-bounding_box.y) > (ssize_t) image->rows))
     {
       /*
         Crop is not within virtual canvas, return 1 pixel transparent image.
@@ -557,11 +561,11 @@ MagickExport Image *CropImage(const Image *image,const RectangleInfo *geometry,
       if (page.y < 0)
         page.y=0;
     }
-  if ((unsigned long) (page.x+page.width) > image->columns)
+  if ((size_t) (page.x+page.width) > image->columns)
     page.width=image->columns-page.x;
   if ((geometry->width != 0) && (page.width > geometry->width))
     page.width=geometry->width;
-  if ((unsigned long) (page.y+page.height) > image->rows)
+  if ((size_t) (page.y+page.height) > image->rows)
     page.height=image->rows-page.y;
   if ((geometry->height != 0) && (page.height > geometry->height))
     page.height=geometry->height;
@@ -581,8 +585,8 @@ MagickExport Image *CropImage(const Image *image,const RectangleInfo *geometry,
     return((Image *) NULL);
   crop_image->page.width=image->page.width;
   crop_image->page.height=image->page.height;
-  if (((long) (bounding_box.x+bounding_box.width) > (long) image->page.width) ||
-      ((long) (bounding_box.y+bounding_box.height) > (long) image->page.height))
+  if (((ssize_t) (bounding_box.x+bounding_box.width) > (ssize_t) image->page.width) ||
+      ((ssize_t) (bounding_box.y+bounding_box.height) > (ssize_t) image->page.height))
     {
       crop_image->page.width=bounding_box.width;
       crop_image->page.height=bounding_box.height;
@@ -596,10 +600,10 @@ MagickExport Image *CropImage(const Image *image,const RectangleInfo *geometry,
   progress=0;
   image_view=AcquireCacheView(image);
   crop_view=AcquireCacheView(crop_image);
-#if defined(MAGICKCORE_OPENMP_SUPPORT)
-  #pragma omp parallel for schedule(static) shared(progress,status)
+#if defined(MAGICKCORE_OPENMP_SUPPORT) 
+  #pragma omp parallel for schedule(dynamic,4) shared(progress,status)
 #endif
-  for (y=0; y < (long) crop_image->rows; y++)
+  for (y=0; y < (ssize_t) crop_image->rows; y++)
   {
     register const IndexPacket
       *restrict indexes;
@@ -624,15 +628,13 @@ MagickExport Image *CropImage(const Image *image,const RectangleInfo *geometry,
         status=MagickFalse;
         continue;
       }
-    (void) CopyMagickMemory(q,p,(size_t) crop_image->columns*sizeof(*q));
     indexes=GetCacheViewVirtualIndexQueue(image_view);
-    if (indexes != (IndexPacket *) NULL)
-      {
-        crop_indexes=GetCacheViewAuthenticIndexQueue(crop_view);
-        if (crop_indexes != (IndexPacket *) NULL)
-          (void) CopyMagickMemory(crop_indexes,indexes,(size_t)
-            crop_image->columns*sizeof(*crop_indexes));
-      }
+    crop_indexes=GetCacheViewAuthenticIndexQueue(crop_view);
+    (void) CopyMagickMemory(q,p,(size_t) crop_image->columns*sizeof(*p));
+    if ((indexes != (IndexPacket *) NULL) &&
+        (crop_indexes != (IndexPacket *) NULL))
+      (void) CopyMagickMemory(crop_indexes,indexes,(size_t) crop_image->columns*
+        sizeof(*crop_indexes));
     if (SyncCacheViewAuthenticPixels(crop_view,exception) == MagickFalse)
       status=MagickFalse;
     if (image->progress_monitor != (MagickProgressMonitor) NULL)
@@ -640,7 +642,7 @@ MagickExport Image *CropImage(const Image *image,const RectangleInfo *geometry,
         MagickBooleanType
           proceed;
 
-#if defined(MAGICKCORE_OPENMP_SUPPORT)
+#if defined(MAGICKCORE_OPENMP_SUPPORT) 
   #pragma omp critical (MagickCore_CropImage)
 #endif
         proceed=SetImageProgress(image,CropImageTag,progress++,image->rows);
@@ -696,13 +698,15 @@ MagickExport Image *ExcerptImage(const Image *image,
   Image
     *excerpt_image;
 
-  long
-    progress,
-    y;
-
   MagickBooleanType
     status;
 
+  MagickOffsetType
+    progress;
+
+  ssize_t
+    y;
+
   /*
     Allocate excerpt image.
   */
@@ -727,7 +731,7 @@ MagickExport Image *ExcerptImage(const Image *image,
 #if defined(MAGICKCORE_OPENMP_SUPPORT)
   #pragma omp parallel for schedule(dynamic,4) shared(progress,status)
 #endif
-  for (y=0; y < (long) excerpt_image->rows; y++)
+  for (y=0; y < (ssize_t) excerpt_image->rows; y++)
   {
     register const PixelPacket
       *restrict p;
@@ -882,13 +886,15 @@ MagickExport Image *FlipImage(const Image *image,ExceptionInfo *exception)
   Image
     *flip_image;
 
-  long
-    progress,
-    y;
-
   MagickBooleanType
     status;
 
+  MagickOffsetType
+    progress;
+
+  ssize_t
+    y;
+
   assert(image != (const Image *) NULL);
   assert(image->signature == MagickSignature);
   if (image->debug != MagickFalse)
@@ -906,9 +912,9 @@ MagickExport Image *FlipImage(const Image *image,ExceptionInfo *exception)
   image_view=AcquireCacheView(image);
   flip_view=AcquireCacheView(flip_image);
 #if defined(MAGICKCORE_OPENMP_SUPPORT)
-  #pragma omp parallel for schedule(static) shared(progress,status)
+  #pragma omp parallel for schedule(dynamic,4) shared(progress,status)
 #endif
-  for (y=0; y < (long) flip_image->rows; y++)
+  for (y=0; y < (ssize_t) flip_image->rows; y++)
   {
     register const IndexPacket
       *restrict indexes;
@@ -925,8 +931,8 @@ MagickExport Image *FlipImage(const Image *image,ExceptionInfo *exception)
     if (status == MagickFalse)
       continue;
     p=GetCacheViewVirtualPixels(image_view,0,y,image->columns,1,exception);
-    q=QueueCacheViewAuthenticPixels(flip_view,0,(long) (flip_image->rows-y-1),
-      flip_image->columns,1,exception);
+    q=QueueCacheViewAuthenticPixels(flip_view,0,(ssize_t) (flip_image->rows-y-
+      1),flip_image->columns,1,exception);
     if ((p == (const PixelPacket *) NULL) || (q == (PixelPacket *) NULL))
       {
         status=MagickFalse;
@@ -948,7 +954,7 @@ MagickExport Image *FlipImage(const Image *image,ExceptionInfo *exception)
         MagickBooleanType
           proceed;
 
-#if defined(MAGICKCORE_OPENMP_SUPPORT)
+#if defined(MAGICKCORE_OPENMP_SUPPORT) 
   #pragma omp critical (MagickCore_FlipImage)
 #endif
         proceed=SetImageProgress(image,FlipImageTag,progress++,image->rows);
@@ -1000,13 +1006,15 @@ MagickExport Image *FlopImage(const Image *image,ExceptionInfo *exception)
   Image
     *flop_image;
 
-  long
-    progress,
-    y;
-
   MagickBooleanType
     status;
 
+  MagickOffsetType
+    progress;
+
+  ssize_t
+    y;
+
   assert(image != (const Image *) NULL);
   assert(image->signature == MagickSignature);
   if (image->debug != MagickFalse)
@@ -1024,9 +1032,9 @@ MagickExport Image *FlopImage(const Image *image,ExceptionInfo *exception)
   image_view=AcquireCacheView(image);
   flop_view=AcquireCacheView(flop_image);
 #if defined(MAGICKCORE_OPENMP_SUPPORT)
-  #pragma omp parallel for schedule(static) shared(progress,status)
+  #pragma omp parallel for schedule(dynamic,4) shared(progress,status)
 #endif
-  for (y=0; y < (long) flop_image->rows; y++)
+  for (y=0; y < (ssize_t) flop_image->rows; y++)
   {
     register const IndexPacket
       *restrict indexes;
@@ -1037,7 +1045,7 @@ MagickExport Image *FlopImage(const Image *image,ExceptionInfo *exception)
     register IndexPacket
       *restrict flop_indexes;
 
-    register long
+    register ssize_t
       x;
 
     register PixelPacket
@@ -1056,7 +1064,7 @@ MagickExport Image *FlopImage(const Image *image,ExceptionInfo *exception)
     q+=flop_image->columns;
     indexes=GetCacheViewVirtualIndexQueue(image_view);
     flop_indexes=GetCacheViewAuthenticIndexQueue(flop_view);
-    for (x=0; x < (long) flop_image->columns; x++)
+    for (x=0; x < (ssize_t) flop_image->columns; x++)
     {
       (*--q)=(*p++);
       if ((indexes != (const IndexPacket *) NULL) &&
@@ -1101,8 +1109,8 @@ MagickExport Image *FlopImage(const Image *image,ExceptionInfo *exception)
 %
 %  The format of the RollImage method is:
 %
-%      Image *RollImage(const Image *image,const long x_offset,
-%        const long y_offset,ExceptionInfo *exception)
+%      Image *RollImage(const Image *image,const ssize_t x_offset,
+%        const ssize_t y_offset,ExceptionInfo *exception)
 %
 %  A description of each parameter follows:
 %
@@ -1117,15 +1125,15 @@ MagickExport Image *FlopImage(const Image *image,ExceptionInfo *exception)
 */
 
 static inline MagickBooleanType CopyImageRegion(Image *destination,
-  const Image *source,const unsigned long columns,const unsigned long rows,
-  const long sx,const long sy,const long dx,const long dy,
+  const Image *source,const size_t columns,const size_t rows,
+  const ssize_t sx,const ssize_t sy,const ssize_t dx,const ssize_t dy,
   ExceptionInfo *exception)
 {
   CacheView
     *source_view,
     *destination_view;
 
-  long
+  ssize_t
     y;
 
   MagickBooleanType
@@ -1137,21 +1145,20 @@ static inline MagickBooleanType CopyImageRegion(Image *destination,
 #if defined(MAGICKCORE_OPENMP_SUPPORT)
   #pragma omp parallel for schedule(dynamic,4) shared(status)
 #endif
-  for (y=0; y < (long) rows; y++)
+  for (y=0; y < (ssize_t) rows; y++)
   {
     MagickBooleanType
       sync;
 
+    register const IndexPacket
+      *restrict indexes;
+
     register const PixelPacket
       *restrict p;
 
     register IndexPacket
-      *restrict indexes,
       *restrict destination_indexes;
 
-    register long
-      x;
-
     register PixelPacket
       *restrict q;
 
@@ -1167,14 +1174,14 @@ static inline MagickBooleanType CopyImageRegion(Image *destination,
         status=MagickFalse;
         continue;
       }
-    indexes=GetCacheViewAuthenticIndexQueue(source_view);
-    for (x=0; x < (long) columns; x++)
-      *q++=(*p++);
+    indexes=GetCacheViewVirtualIndexQueue(source_view);
+    (void) CopyMagickMemory(q,p,(size_t) columns*sizeof(*p));
     if (indexes != (IndexPacket *) NULL)
       {
         destination_indexes=GetCacheViewAuthenticIndexQueue(destination_view);
-        for (x=0; x < (long) columns; x++)
-          destination_indexes[x]=indexes[x];
+        if (destination_indexes != (IndexPacket *) NULL)
+          (void) CopyMagickMemory(destination_indexes,indexes,(size_t)
+            columns*sizeof(*indexes));
       }
     sync=SyncCacheViewAuthenticPixels(destination_view,exception);
     if (sync == MagickFalse)
@@ -1185,8 +1192,8 @@ static inline MagickBooleanType CopyImageRegion(Image *destination,
   return(status);
 }
 
-MagickExport Image *RollImage(const Image *image,const long x_offset,
-  const long y_offset,ExceptionInfo *exception)
+MagickExport Image *RollImage(const Image *image,const ssize_t x_offset,
+  const ssize_t y_offset,ExceptionInfo *exception)
 {
 #define RollImageTag  "Roll/Image"
 
@@ -1214,26 +1221,26 @@ MagickExport Image *RollImage(const Image *image,const long x_offset,
   offset.x=x_offset;
   offset.y=y_offset;
   while (offset.x < 0)
-    offset.x+=image->columns;
-  while (offset.x >= (long) image->columns)
-    offset.x-=image->columns;
+    offset.x+=(ssize_t) image->columns;
+  while (offset.x >= (ssize_t) image->columns)
+    offset.x-=(ssize_t) image->columns;
   while (offset.y < 0)
-    offset.y+=image->rows;
-  while (offset.y >= (long) image->rows)
-    offset.y-=image->rows;
+    offset.y+=(ssize_t) image->rows;
+  while (offset.y >= (ssize_t) image->rows)
+    offset.y-=(ssize_t) image->rows;
   /*
     Roll image.
   */
-  status=CopyImageRegion(roll_image,image,(unsigned long) offset.x,
-    (unsigned long) offset.y,(long) image->columns-offset.x,(long) image->rows-
+  status=CopyImageRegion(roll_image,image,(size_t) offset.x,
+    (size_t) offset.y,(ssize_t) image->columns-offset.x,(ssize_t) image->rows-
     offset.y,0,0,exception);
   (void) SetImageProgress(image,RollImageTag,0,3);
   status|=CopyImageRegion(roll_image,image,image->columns-offset.x,
-    (unsigned long) offset.y,0,(long) image->rows-offset.y,offset.x,0,
+    (size_t) offset.y,0,(ssize_t) image->rows-offset.y,offset.x,0,
     exception);
   (void) SetImageProgress(image,RollImageTag,1,3);
-  status|=CopyImageRegion(roll_image,image,(unsigned long) offset.x,image->rows-
-    offset.y,(long) image->columns-offset.x,0,0,offset.y,exception);
+  status|=CopyImageRegion(roll_image,image,(size_t) offset.x,image->rows-
+    offset.y,(ssize_t) image->columns-offset.x,0,0,offset.y,exception);
   (void) SetImageProgress(image,RollImageTag,2,3);
   status|=CopyImageRegion(roll_image,image,image->columns-offset.x,image->rows-
     offset.y,0,0,offset.x,offset.y,exception);
@@ -1297,15 +1304,15 @@ MagickExport Image *ShaveImage(const Image *image,
   SetGeometry(image,&geometry);
   geometry.width-=2*shave_info->width;
   geometry.height-=2*shave_info->height;
-  geometry.x=(long) shave_info->width+image->page.x;
-  geometry.y=(long) shave_info->height+image->page.y;
+  geometry.x=(ssize_t) shave_info->width+image->page.x;
+  geometry.y=(ssize_t) shave_info->height+image->page.y;
   shave_image=CropImage(image,&geometry,exception);
   if (shave_image == (Image *) NULL)
     return((Image *) NULL);
   shave_image->page.width-=2*shave_info->width;
   shave_image->page.height-=2*shave_info->height;
-  shave_image->page.x-=shave_info->width;
-  shave_image->page.y-=shave_info->height;
+  shave_image->page.x-=(ssize_t) shave_info->width;
+  shave_image->page.y-=(ssize_t) shave_info->height;
   return(shave_image);
 }
 
@@ -1350,20 +1357,22 @@ MagickExport Image *SpliceImage(const Image *image,
   Image
     *splice_image;
 
-  long
-    progress,
-    y;
-
   MagickBooleanType
     proceed,
     status;
 
+  MagickOffsetType
+    progress;
+
   RectangleInfo
     splice_geometry;
 
-  register long
+  register ssize_t
     i;
 
+  ssize_t
+    y;
+
   /*
     Allocate splice image.
   */
@@ -1397,47 +1406,47 @@ MagickExport Image *SpliceImage(const Image *image,
       break;
     case NorthGravity:
     {
-      splice_geometry.x+=splice_geometry.width/2;
+      splice_geometry.x+=(ssize_t) splice_geometry.width/2;
       break;
     }
     case NorthEastGravity:
     {
-      splice_geometry.x+=splice_geometry.width;
+      splice_geometry.x+=(ssize_t) splice_geometry.width;
       break;
     }
     case WestGravity:
     {
-      splice_geometry.y+=splice_geometry.width/2;
+      splice_geometry.y+=(ssize_t) splice_geometry.width/2;
       break;
     }
     case StaticGravity:
     case CenterGravity:
     {
-      splice_geometry.x+=splice_geometry.width/2;
-      splice_geometry.y+=splice_geometry.height/2;
+      splice_geometry.x+=(ssize_t) splice_geometry.width/2;
+      splice_geometry.y+=(ssize_t) splice_geometry.height/2;
       break;
     }
     case EastGravity:
     {
-      splice_geometry.x+=splice_geometry.width;
-      splice_geometry.y+=splice_geometry.height/2;
+      splice_geometry.x+=(ssize_t) splice_geometry.width;
+      splice_geometry.y+=(ssize_t) splice_geometry.height/2;
       break;
     }
     case SouthWestGravity:
     {
-      splice_geometry.y+=splice_geometry.height;
+      splice_geometry.y+=(ssize_t) splice_geometry.height;
       break;
     }
     case SouthGravity:
     {
-      splice_geometry.x+=splice_geometry.width/2;
-      splice_geometry.y+=splice_geometry.height;
+      splice_geometry.x+=(ssize_t) splice_geometry.width/2;
+      splice_geometry.y+=(ssize_t) splice_geometry.height;
       break;
     }
     case SouthEastGravity:
     {
-      splice_geometry.x+=splice_geometry.width;
-      splice_geometry.y+=splice_geometry.height;
+      splice_geometry.x+=(ssize_t) splice_geometry.width;
+      splice_geometry.y+=(ssize_t) splice_geometry.height;
       break;
     }
   }
@@ -1452,7 +1461,7 @@ MagickExport Image *SpliceImage(const Image *image,
 #if defined(MAGICKCORE_OPENMP_SUPPORT)
   #pragma omp parallel for schedule(dynamic,4) shared(progress,status)
 #endif
-  for (y=0; y < (long) splice_geometry.y; y++)
+  for (y=0; y < (ssize_t) splice_geometry.y; y++)
   {
     register const PixelPacket
       *restrict p;
@@ -1461,7 +1470,7 @@ MagickExport Image *SpliceImage(const Image *image,
       *restrict indexes,
       *restrict splice_indexes;
 
-    register long
+    register ssize_t
       x;
 
     register PixelPacket
@@ -1492,9 +1501,9 @@ MagickExport Image *SpliceImage(const Image *image,
       p++;
       q++;
     }
-    for ( ; x < (long) (splice_geometry.x+splice_geometry.width); x++)
+    for ( ; x < (ssize_t) (splice_geometry.x+splice_geometry.width); x++)
       q++;
-    for ( ; x < (long) splice_image->columns; x++)
+    for ( ; x < (ssize_t) splice_image->columns; x++)
     {
       SetRedPixelComponent(q,GetRedPixelComponent(p));
       SetGreenPixelComponent(q,GetGreenPixelComponent(p));
@@ -1509,7 +1518,8 @@ MagickExport Image *SpliceImage(const Image *image,
     }
     if (SyncCacheViewAuthenticPixels(splice_view,exception) == MagickFalse)
       status=MagickFalse;
-    proceed=SetImageProgress(image,SpliceImageTag,y,splice_image->rows);
+    proceed=SetImageProgress(image,SpliceImageTag,(MagickOffsetType) y,
+      splice_image->rows);
     if (image->progress_monitor != (MagickProgressMonitor) NULL)
       {
         MagickBooleanType
@@ -1527,8 +1537,8 @@ MagickExport Image *SpliceImage(const Image *image,
 #if defined(MAGICKCORE_OPENMP_SUPPORT)
   #pragma omp parallel for schedule(dynamic,4) shared(progress,status)
 #endif
-  for (y=(long) (splice_geometry.y+splice_geometry.height);
-       y < (long) splice_image->rows; y++)
+  for (y=(ssize_t) (splice_geometry.y+splice_geometry.height);
+       y < (ssize_t) splice_image->rows; y++)
   {
     register const PixelPacket
       *restrict p;
@@ -1537,7 +1547,7 @@ MagickExport Image *SpliceImage(const Image *image,
       *restrict indexes,
       *restrict splice_indexes;
 
-    register long
+    register ssize_t
       x;
 
     register PixelPacket
@@ -1545,7 +1555,7 @@ MagickExport Image *SpliceImage(const Image *image,
 
     if (status == MagickFalse)
       continue;
-    p=GetCacheViewVirtualPixels(image_view,0,y-splice_geometry.height,
+    p=GetCacheViewVirtualPixels(image_view,0,y-(ssize_t) splice_geometry.height,
       image->columns,1,exception);
     q=QueueCacheViewAuthenticPixels(splice_view,0,y,splice_image->columns,1,
       exception);
@@ -1569,9 +1579,9 @@ MagickExport Image *SpliceImage(const Image *image,
       p++;
       q++;
     }
-    for ( ; x < (long) (splice_geometry.x+splice_geometry.width); x++)
+    for ( ; x < (ssize_t) (splice_geometry.x+splice_geometry.width); x++)
       q++;
-    for ( ; x < (long) splice_image->columns; x++)
+    for ( ; x < (ssize_t) splice_image->columns; x++)
     {
       SetRedPixelComponent(q,GetRedPixelComponent(p));
       SetGreenPixelComponent(q,GetGreenPixelComponent(p));
@@ -1639,14 +1649,14 @@ MagickExport Image *SpliceImage(const Image *image,
 %      final size of the image.
 %
 */
-static inline long MagickRound(MagickRealType x)
+static inline ssize_t MagickRound(MagickRealType x)
 {
   /*
     Round the fraction to nearest integer.
   */
   if (x >= 0.0)
-    return((long) (x+0.5));
-  return((long) (x-0.5));
+    return((ssize_t) (x+0.5));
+  return((ssize_t) (x-0.5));
 }
 
 MagickExport MagickBooleanType TransformImage(Image **image,
@@ -1657,7 +1667,7 @@ MagickExport MagickBooleanType TransformImage(Image **image,
     *resize_image,
     *transform_image;
 
-  long
+  ssize_t
     x,
     y;
 
@@ -1667,7 +1677,7 @@ MagickExport MagickBooleanType TransformImage(Image **image,
   RectangleInfo
     geometry;
 
-  unsigned long
+  size_t
     height,
     width;
 
@@ -1725,40 +1735,42 @@ MagickExport MagickBooleanType TransformImage(Image **image,
           {
             if ((flags & AspectValue) == 0)
               {
-                crop.y=(long) MagickRound((MagickRealType) (offset.y-
+                crop.y=(ssize_t) MagickRound((MagickRealType) (offset.y-
                   (geometry.y > 0 ? 0 : geometry.y)));
                 offset.y+=delta.y;
-                crop.height=(unsigned long) MagickRound((MagickRealType)
+                crop.height=(size_t) MagickRound((MagickRealType)
                   (offset.y+(geometry.y < 0 ? 0 : geometry.y)));
               }
             else
               {
-                crop.y=(long) MagickRound((MagickRealType) (offset.y-
+                crop.y=(ssize_t) MagickRound((MagickRealType) (offset.y-
                   (geometry.y > 0 ? geometry.y : 0)));
                 offset.y+=delta.y;
-                crop.height=(unsigned long) MagickRound((MagickRealType)
+                crop.height=(size_t) MagickRound((MagickRealType)
                   (offset.y+(geometry.y < 0 ? geometry.y : 0)));
               }
             crop.height-=crop.y;
+            crop.y+=transform_image->page.y;
             for (offset.x=0; offset.x < (double) width; )
             {
               if ((flags & AspectValue) == 0)
                 {
-                  crop.x=(long) MagickRound((MagickRealType) (offset.x-
+                  crop.x=(ssize_t) MagickRound((MagickRealType) (offset.x-
                     (geometry.x > 0 ? 0 : geometry.x)));
                   offset.x+=+delta.x;
-                  crop.width=(unsigned long) MagickRound((MagickRealType)
+                  crop.width=(size_t) MagickRound((MagickRealType)
                     (offset.x+(geometry.x < 0 ? 0 : geometry.x)));
                 }
               else
                 {
-                  crop.x=(long) MagickRound((MagickRealType) (offset.x-
+                  crop.x=(ssize_t) MagickRound((MagickRealType) (offset.x-
                     (geometry.x > 0 ? geometry.x : 0)));
                   offset.x+=+delta.x;
-                  crop.width=(unsigned long) MagickRound((MagickRealType)
+                  crop.width=(size_t) MagickRound((MagickRealType)
                     (offset.x+(geometry.x < 0 ? geometry.x : 0)));
                 }
               crop.width-=crop.x;
+              crop.x+=transform_image->page.x;
               next=CropImage(transform_image,&crop,&(*image)->exception);
               if (next == (Image *) NULL)
                 break;
@@ -1818,12 +1830,12 @@ MagickExport MagickBooleanType TransformImage(Image **image,
              proceed=MagickTrue;
              i=0;
              number_images=0;
-             for (y=0; y < (long) transform_image->page.height; y+=height)
-               for (x=0; x < (long) transform_image->page.width; x+=width)
+             for (y=0; y < (ssize_t) transform_image->page.height; y+=(ssize_t) height)
+               for (x=0; x < (ssize_t) transform_image->page.width; x+=(ssize_t) width)
                  number_images++;
-             for (y=0; y < (long) transform_image->page.height; y+=height)
+             for (y=0; y < (ssize_t) transform_image->page.height; y+=(ssize_t) height)
              {
-               for (x=0; x < (long) transform_image->page.width; x+=width)
+               for (x=0; x < (ssize_t) transform_image->page.width; x+=(ssize_t) width)
                {
                  progress_monitor=SetImageProgressMonitor(transform_image,
                    (MagickProgressMonitor) NULL,transform_image->client_data);
@@ -1842,7 +1854,14 @@ MagickExport MagickBooleanType TransformImage(Image **image,
                    break;
                  (void) SetImageProgressMonitor(next,progress_monitor,
                    next->client_data);
-                 AppendImageToList(&crop_image,next);
+                 if (crop_image == (Image *) NULL)
+                   crop_image=next;
+                 else
+                   {
+                     next->previous=crop_image;
+                     crop_image->next=next;
+                     crop_image=crop_image->next;
+                   }
                }
                if (next == (Image *) NULL)
                  break;
@@ -1918,7 +1937,7 @@ MagickExport MagickBooleanType TransformImages(Image **images,
   MagickStatusType
     status;
 
-  register long
+  register ssize_t
     i;
 
   assert(images != (Image **) NULL);
@@ -1978,16 +1997,18 @@ MagickExport Image *TransposeImage(const Image *image,ExceptionInfo *exception)
   Image
     *transpose_image;
 
-  long
-    progress,
-    y;
-
   MagickBooleanType
     status;
 
+  MagickOffsetType
+    progress;
+
   RectangleInfo
     page;
 
+  ssize_t
+    y;
+
   assert(image != (const Image *) NULL);
   assert(image->signature == MagickSignature);
   if (image->debug != MagickFalse)
@@ -2008,7 +2029,7 @@ MagickExport Image *TransposeImage(const Image *image,ExceptionInfo *exception)
 #if defined(MAGICKCORE_OPENMP_SUPPORT)
   #pragma omp parallel for schedule(dynamic,4) shared(progress,status)
 #endif
-  for (y=0; y < (long) image->rows; y++)
+  for (y=0; y < (ssize_t) image->rows; y++)
   {
     register const PixelPacket
       *restrict p;
@@ -2022,10 +2043,10 @@ MagickExport Image *TransposeImage(const Image *image,ExceptionInfo *exception)
 
     if (status == MagickFalse)
       continue;
-    p=GetCacheViewVirtualPixels(image_view,0,(long) image->rows-y-1,
+    p=GetCacheViewVirtualPixels(image_view,0,(ssize_t) image->rows-y-1,
       image->columns,1,exception);
-    q=QueueCacheViewAuthenticPixels(transpose_view,(long) (image->rows-y-1),0,
-      1,transpose_image->rows,exception);
+    q=QueueCacheViewAuthenticPixels(transpose_view,(ssize_t) (image->rows-y-1),
+      0,1,transpose_image->rows,exception);
     if ((p == (const PixelPacket *) NULL) || (q == (PixelPacket *) NULL))
       {
         status=MagickFalse;
@@ -2063,7 +2084,7 @@ MagickExport Image *TransposeImage(const Image *image,ExceptionInfo *exception)
   Swap(page.width,page.height);
   Swap(page.x,page.y);
   if (page.width != 0)
-    page.x=(long) (page.width-transpose_image->columns-page.x);
+    page.x=(ssize_t) (page.width-transpose_image->columns-page.x);
   transpose_image->page=page;
   if (status == MagickFalse)
     transpose_image=DestroyImage(transpose_image);
@@ -2106,16 +2127,18 @@ MagickExport Image *TransverseImage(const Image *image,ExceptionInfo *exception)
   Image
     *transverse_image;
 
-  long
-    progress,
-    y;
-
   MagickBooleanType
     status;
 
+  MagickOffsetType
+    progress;
+
   RectangleInfo
     page;
 
+  ssize_t
+    y;
+
   assert(image != (const Image *) NULL);
   assert(image->signature == MagickSignature);
   if (image->debug != MagickFalse)
@@ -2136,7 +2159,7 @@ MagickExport Image *TransverseImage(const Image *image,ExceptionInfo *exception)
 #if defined(MAGICKCORE_OPENMP_SUPPORT)
   #pragma omp parallel for schedule(dynamic,4) shared(progress,status)
 #endif
-  for (y=0; y < (long) image->rows; y++)
+  for (y=0; y < (ssize_t) image->rows; y++)
   {
     MagickBooleanType
       sync;
@@ -2148,7 +2171,7 @@ MagickExport Image *TransverseImage(const Image *image,ExceptionInfo *exception)
       *restrict transverse_indexes,
       *restrict indexes;
 
-    register long
+    register ssize_t
       x;
 
     register PixelPacket
@@ -2157,7 +2180,7 @@ MagickExport Image *TransverseImage(const Image *image,ExceptionInfo *exception)
     if (status == MagickFalse)
       continue;
     p=GetCacheViewVirtualPixels(image_view,0,y,image->columns,1,exception);
-    q=QueueCacheViewAuthenticPixels(transverse_view,(long) (image->rows-y-
+    q=QueueCacheViewAuthenticPixels(transverse_view,(ssize_t) (image->rows-y-
       1),0,1,transverse_image->rows,exception);
     if ((p == (const PixelPacket *) NULL) || (q == (PixelPacket *) NULL))
       {
@@ -2165,14 +2188,14 @@ MagickExport Image *TransverseImage(const Image *image,ExceptionInfo *exception)
         continue;
       }
     q+=image->columns;
-    for (x=0; x < (long) image->columns; x++)
+    for (x=0; x < (ssize_t) image->columns; x++)
       *--q=(*p++);
     indexes=GetCacheViewAuthenticIndexQueue(image_view);
     if (indexes != (IndexPacket *) NULL)
       {
         transverse_indexes=GetCacheViewAuthenticIndexQueue(transverse_view);
         if (transverse_indexes != (IndexPacket *) NULL)
-          for (x=0; x < (long) image->columns; x++)
+          for (x=0; x < (ssize_t) image->columns; x++)
             transverse_indexes[image->columns-x-1]=indexes[x];
       }
     sync=SyncCacheViewAuthenticPixels(transverse_view,exception);
@@ -2199,7 +2222,7 @@ MagickExport Image *TransverseImage(const Image *image,ExceptionInfo *exception)
   Swap(page.width,page.height);
   Swap(page.x,page.y);
   if (page.height != 0)
-    page.y=(long) (page.height-transverse_image->rows-page.y);
+    page.y=(ssize_t) (page.height-transverse_image->rows-page.y);
   transverse_image->page=page;
   if (status == MagickFalse)
     transverse_image=DestroyImage(transverse_image);
