@@ -137,6 +137,7 @@ static MagickBooleanType CompareUsage(void)
       "-set attribute value set an image attribute",
       "-quality value       JPEG/MIFF/PNG compression level",
       "-size geometry       width and height of image",
+      "-subimage-search     search for subimage",
       "-transparent-color color",
       "                     transparent color",
       "-type type           image type",
@@ -225,13 +226,10 @@ WandExport MagickBooleanType CompareImageCommand(ImageInfo *image_info,
   ImageStack
     image_stack[MaxImageStackDepth+1];
 
-  ssize_t
-    j,
-    k;
-
   MagickBooleanType
     fire,
-    pend;
+    pend,
+    subimage_search;
 
   MagickStatusType
     status;
@@ -245,6 +243,10 @@ WandExport MagickBooleanType CompareImageCommand(ImageInfo *image_info,
   register ssize_t
     i;
 
+  ssize_t
+    j,
+    k;
+
   /*
     Set defaults.
   */
@@ -282,6 +284,7 @@ WandExport MagickBooleanType CompareImageCommand(ImageInfo *image_info,
   pend=MagickFalse;
   reconstruct_image=NewImageList();
   status=MagickTrue;
+  subimage_search=MagickFalse;
   /*
     Compare an image.
   */
@@ -836,6 +839,16 @@ WandExport MagickBooleanType CompareImageCommand(ImageInfo *image_info,
               ThrowCompareInvalidArgumentException(option,argv[i]);
             break;
           }
+        if (LocaleCompare("subimage-search",option+1) == 0)
+          {
+            if (*option == '+')
+              {
+                subimage_search=MagickFalse;
+                break;
+              }
+            subimage_search=MagickTrue;
+            break;
+          }
         ThrowCompareException(OptionError,"UnrecognizedOption",option)
       }
       case 't':
@@ -920,16 +933,22 @@ WandExport MagickBooleanType CompareImageCommand(ImageInfo *image_info,
     ThrowCompareException(OptionError,"MissingAnImageFilename",argv[i]);
   image=GetImageFromList(image,0);
   reconstruct_image=GetImageFromList(image,1);
-  similarity_image=SimilarityImage(image,reconstruct_image,&offset,
-    &similarity_metric,exception);
-  if (similarity_metric > dissimilarity_threshold)
-    ThrowCompareException(ImageError,"ImagesTooDissimilar",image->filename);
+  if (subimage_search != MagickFalse)
+    {
+      similarity_image=SimilarityImage(image,reconstruct_image,&offset,
+        &similarity_metric,exception);
+      if (similarity_metric > dissimilarity_threshold)
+        ThrowCompareException(ImageError,"ImagesTooDissimilar",image->filename);
+    }
   if ((reconstruct_image->columns == image->columns) &&
       (reconstruct_image->rows == image->rows))
     difference_image=CompareImageChannels(image,reconstruct_image,channels,
       metric,&distortion,exception);
   else
-    if (similarity_image != (Image *) NULL)
+    if (similarity_image == (Image *) NULL)
+      ThrowCompareException(OptionError,"ImageWidthsOrHeightsDiffer",
+        image->filename)
+    else
       {
         Image
           *composite_image;
@@ -954,9 +973,7 @@ WandExport MagickBooleanType CompareImageCommand(ImageInfo *image_info,
               }
             composite_image=DestroyImage(composite_image);
           }
-        if (difference_image == (Image *) NULL)
-          similarity_image=DestroyImage(similarity_image);
-        else
+        if (difference_image != (Image *) NULL)
           {
             AppendImageToList(&difference_image,similarity_image);
             similarity_image=(Image *) NULL;
